@@ -14,7 +14,6 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-macros.h>
-#include <libgnomeui/gnome-window-icon.h>
 #include "panel-marshal.h"
 #include "panel-widget.h"
 #include "basep-widget.h"
@@ -32,7 +31,6 @@
 
 #undef BASEP_WIDGET_DEBUG
 
-extern gboolean panel_applet_in_drag;
 extern GSList *panel_list;
 
 extern int panels_to_sync;
@@ -62,6 +60,7 @@ static void basep_widget_realize (GtkWidget *w);
 static void basep_widget_map (GtkWidget *w);
 static gboolean basep_enter_notify (GtkWidget *widget, GdkEventCrossing *event);
 static gboolean basep_leave_notify (GtkWidget *widget, GdkEventCrossing *event);
+static gboolean basep_key_press (GtkWidget *widget, GdkEventKey *event);
 static void basep_style_set (GtkWidget *widget, GtkStyle *previous_style);
 static void basep_widget_destroy (GtkObject *o);
 
@@ -143,6 +142,7 @@ basep_widget_class_init (BasePWidgetClass *klass)
 	widget_class->map = basep_widget_map;
 	widget_class->enter_notify_event = basep_enter_notify;
 	widget_class->leave_notify_event = basep_leave_notify;
+	widget_class->key_press_event = basep_key_press;
 	widget_class->style_set = basep_style_set;
 
 	gtk_object_class->destroy = basep_widget_destroy;
@@ -190,7 +190,7 @@ basep_widget_class_init (BasePWidgetClass *klass)
 			      G_STRUCT_OFFSET (BasePWidgetClass, type_change),
 			      NULL,
 			      NULL,
-			      panel_marshal_VOID__ENUM,
+			      g_cclosure_marshal_VOID__ENUM,
 			      G_TYPE_NONE,
 			      1,
 			      PANEL_TYPE_OBJECT_TYPE); */
@@ -213,7 +213,7 @@ basep_widget_class_init (BasePWidgetClass *klass)
 				G_STRUCT_OFFSET (BasePWidgetClass, mode_change),
 				NULL,
 				NULL,
-				panel_marshal_VOID__ENUM,
+				g_cclosure_marshal_VOID__ENUM,
 				G_TYPE_NONE,
 				1,
 				PANEL_TYPE_BASE_PMODE);
@@ -225,7 +225,7 @@ basep_widget_class_init (BasePWidgetClass *klass)
 				G_STRUCT_OFFSET (BasePWidgetClass, state_change),
 				NULL,
 				NULL,
-				panel_marshal_VOID__ENUM,
+				g_cclosure_marshal_VOID__ENUM,
 				G_TYPE_NONE,
 				1,
 				PANEL_TYPE_BASE_PSTATE);
@@ -572,7 +572,9 @@ basep_widget_move_focus_out (BasePWidget *basep)
 		gtk_window_present (GTK_WINDOW (parent));
 		gtk_widget_grab_focus (drawer->button);
 	} else {
-		panel_widget_focus (PANEL_WIDGET (basep->panel));
+		PanelWidget *panel = PANEL_WIDGET (basep->panel);
+
+		panel_widget_focus (panel);
 	}
 }
 
@@ -825,6 +827,24 @@ basep_pos_instance_init (BasePPos *pos)
 	return;
 }
 
+static gboolean 
+basep_key_press (GtkWidget *widget, GdkEventKey *event)
+{
+	BasePWidget *basep;
+	PanelWidget *panel;
+
+	g_return_val_if_fail (BASEP_IS_WIDGET (widget), FALSE);
+
+	basep = BASEP_WIDGET (widget);
+
+	g_return_val_if_fail (PANEL_IS_WIDGET (basep->panel), FALSE);
+
+	panel = PANEL_WIDGET (basep->panel);
+	
+	panel_widget_save_key_event (panel, event);
+	return GTK_WIDGET_CLASS (basep_widget_parent_class)->key_press_event (widget, event);
+}
+ 
 static gboolean
 basep_leave_notify (GtkWidget *widget,
 		    GdkEventCrossing *event)
@@ -870,9 +890,6 @@ basep_enter_notify (GtkWidget *widget,
 		basep_widget_queue_autoshow (basep);
 	}  
 
-	if (global_config.autoraise)
-		gdk_window_raise (GTK_WIDGET(basep)->window);
-
 	if (GTK_WIDGET_CLASS (basep_widget_parent_class)->enter_notify_event)
 		return GTK_WIDGET_CLASS (basep_widget_parent_class)->enter_notify_event (widget, event);
 	else
@@ -963,23 +980,20 @@ basep_widget_do_hiding(BasePWidget *basep, PanelOrient hide_orient,
 	g_return_if_fail(BASEP_IS_WIDGET(basep));
 
 	switch (animation_step) {
-		case PANEL_SPEED_SLOW:
-				step = PANEL_SLOW_STEP_SIZE;
-				break;
-		case PANEL_SPEED_MEDIUM:
-				step = PANEL_MEDIUM_STEP_SIZE;
-				break;
-		case PANEL_SPEED_FAST:
-				step = PANEL_FAST_STEP_SIZE;
-				break;
-		default:
-				step = PANEL_MEDIUM_STEP_SIZE; 
+	case PANEL_SPEED_SLOW:
+		step = PANEL_SLOW_STEP_SIZE;
+		break;
+	case PANEL_SPEED_MEDIUM:
+		step = PANEL_MEDIUM_STEP_SIZE;
+		break;
+	case PANEL_SPEED_FAST:
+		step = PANEL_FAST_STEP_SIZE;
+		break;
+	default:
+		step = PANEL_MEDIUM_STEP_SIZE; 
+		break;
 	}
 
-#ifdef BASEP_WIDGET_DEBUG
-	g_warning ("do_hiding with step %d", step);
-#endif
-
 	wid = GTK_WIDGET(basep);
 	
 	ox = x = basep->shown_alloc.x;
@@ -1023,16 +1037,13 @@ basep_widget_do_hiding(BasePWidget *basep, PanelOrient hide_orient,
 		break;
 	}
 
-	if(global_config.enable_animations && step != 0) {
+	if (global_config.enable_animations && step != 0) {
 		GTimeVal tval;
 		long start_secs;
 		long start_time;
 		long end_time;
 		long cur_time;
 
-#ifdef BASEP_WIDGET_DEBUG
-	printf ("We are sliding the panel since animations are turned ON\n");
-#endif
 		g_get_current_time(&tval);
 		
 		start_secs = tval.tv_sec;
@@ -1137,7 +1148,7 @@ basep_widget_do_showing(BasePWidget *basep, PanelOrient hide_orient,
 		break;
 	}
 	
-	if(global_config.enable_animations && step != 0) {
+	if (global_config.enable_animations && step != 0) {
 		int i;
 		GTimeVal tval;
 		long start_secs;
@@ -1625,17 +1636,8 @@ basep_widget_construct (gchar *panel_id,
 {
 	BasePPosClass *klass = basep_widget_get_pos_class (basep);
 	int x = 0, y = 0;
-	char *panel_icon;
 	GList *focus_chain = NULL;
 
-	panel_icon = panel_pixmap_discovery ("gnome-panel.png", 
-					     FALSE);
-
-	if (panel_icon) {
-		gnome_window_icon_set_default_from_file (panel_icon);
-		g_free (panel_icon);
-	}
-
 	basep->panel = panel_widget_new(panel_id,
 					packed,
 					orient,
