@@ -97,8 +97,8 @@ translate_to(GtkWidget *from, GtkWidget *to, int *x, int *y)
 {
 	while (from != to) {
 		if (!GTK_WIDGET_NO_WINDOW (from)) {
-			*x += from->allocation.x;
-			*y += from->allocation.y;
+			*x += MAX (from->allocation.x, 0);
+			*y += MAX (from->allocation.y, 0);
 		}
 		from = from->parent;
 	}
@@ -115,8 +115,13 @@ get_frame(BasePWidget *basep)
 }
 
 static void
-calculate_overlay_geometry (PanelWidget *panel, GtkWidget *parent,
-			    GtkWidget *applet, int *x, int *y, int *w, int *h)
+calculate_overlay_geometry (PanelWidget *panel,
+			    GtkWidget   *parent,
+			    GtkWidget   *applet,
+			    int         *x,
+			    int         *y,
+			    int         *w,
+			    int         *h)
 {
 	*x = applet->allocation.x;
 	*y = applet->allocation.y;
@@ -125,19 +130,6 @@ calculate_overlay_geometry (PanelWidget *panel, GtkWidget *parent,
 
 	translate_to (GTK_WIDGET(panel), parent, x, y);
 
-	/* when not shown, things are somewhat weird, and we try to put the
-	 * window completely off as it can't be clickable anyway */
-	/* XXX: These window thingies should really be unmapped in hidden
-	 * case, or something like that, this is ugly, but who gives a fuck,
-	 * this is all going to be rewritten soon (famous last words?) */
-	if(BASEP_IS_WIDGET(parent) &&
-	   BASEP_WIDGET(parent)->state != BASEP_SHOWN &&
-	   BASEP_WIDGET(parent)->state != BASEP_AUTO_HIDDEN) {
-		*x = parent->requisition.width + 1;
-		*y = parent->requisition.height + 1;
-		return;
-	}
-
 	if(panel->orient == GTK_ORIENTATION_HORIZONTAL) {
 		if (applet->allocation.x > panel->size) {
 			*x = parent->requisition.width + 1;
@@ -149,7 +141,8 @@ calculate_overlay_geometry (PanelWidget *panel, GtkWidget *parent,
 		/* we use the requisition, since allocation might have not
 		   yet happened if we are inside the allocation, anyway
 		   they are the same for basep */
-		*h = parent->requisition.height;
+		if(*h < parent->requisition.height)
+			*h = parent->requisition.height;
 
 		if ((*w + applet->allocation.x) > panel->size) {
 			*w = panel->size - applet->allocation.x;
@@ -179,7 +172,8 @@ calculate_overlay_geometry (PanelWidget *panel, GtkWidget *parent,
 		}
 
 		*x = 0;
-		*w = parent->requisition.width;
+		if(*w < parent->requisition.width)
+			*w = parent->requisition.width;
 
 		if ((*h + applet->allocation.y) > panel->size) {
 			*h = panel->size - applet->allocation.y;
