@@ -13,11 +13,8 @@
 #include <config.h>
 #include <unistd.h>
 #include <string.h>
-#include <time.h>
 
 #include <libgnome/libgnome.h>
-#include <libgnomeui/gnome-window-icon.h>
-#include <gconf/gconf-client.h>
 #include <gdk/gdkkeysyms.h>
 
 /* Yes, yes I know, now bugger off ... */
@@ -37,17 +34,14 @@
 #include "drawer-widget.h"
 #include "gnome-run.h"
 #include "multiscreen-stuff.h"
-#include "panel-gconf.h"
 #include "panel-marshal.h"
 
 #define ICON_SIZE 20
+#define FOOBAR_MENU_FLAGS (MAIN_MENU_SYSTEM | MAIN_MENU_KDE_SUB | MAIN_MENU_DISTRIBUTION_SUB)
 
 extern GlobalConfig global_config;
 extern GSList *panel_list;
 
-#define FORMAT_ITEM_NUM 4
-static GtkWidget *format_items[FORMAT_ITEM_NUM] = { NULL };
-
 extern GtkTooltips *panel_tooltips;
 
 static void foobar_widget_class_init	(FoobarWidgetClass *klass);
@@ -60,6 +54,9 @@ static gboolean foobar_leave_notify	(GtkWidget *widget,
 					 GdkEventCrossing *event);
 static gboolean foobar_enter_notify	(GtkWidget *widget,
 					 GdkEventCrossing *event);
+static gboolean foobar_key_press	(GtkWidget *widget,
+					 GdkEventKey *event);
+static void foobar_widget_move_focus_out   (FoobarWidget *foo);
 static void foobar_widget_move_focus_out   (FoobarWidget *foo);
 static void append_task_menu (FoobarWidget *foo, GtkMenuShell *menu_bar);
 static void setup_task_menu (FoobarWidget *foo);
@@ -114,6 +111,7 @@ foobar_widget_class_init (FoobarWidgetClass *klass)
 	widget_class->size_allocate = foobar_widget_size_allocate;
 	widget_class->enter_notify_event = foobar_enter_notify;
 	widget_class->leave_notify_event = foobar_leave_notify;
+	widget_class->key_press_event = foobar_key_press;
 
 	klass->move_focus_out = foobar_widget_move_focus_out;
 
@@ -128,8 +126,8 @@ foobar_widget_class_init (FoobarWidgetClass *klass)
 		g_signal_new	("move_focus_out",
 				G_TYPE_FROM_CLASS (object_class),
 				G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-				G_STRUCT_OFFSET (FoobarWidgetClass, move_focus_out),				NULL,
-                                NULL,
+				G_STRUCT_OFFSET (FoobarWidgetClass, move_focus_out),
+				NULL, NULL,
                                 panel_marshal_VOID__VOID,
 				G_TYPE_NONE,
 				0);
@@ -168,6 +166,24 @@ pixmap_menu_item_new (const char *text, const char *try_file, gboolean force_ima
 	return item;
 }
 
+static gboolean
+foobar_key_press (GtkWidget *widget, GdkEventKey *event)
+{
+ 	FoobarWidget *foobar;
+	PanelWidget *panel;
+
+	g_return_val_if_fail (FOOBAR_IS_WIDGET (widget), FALSE);
+
+	foobar = FOOBAR_WIDGET (widget);
+
+	g_return_val_if_fail (PANEL_IS_WIDGET (foobar->panel), FALSE);
+
+	panel = PANEL_WIDGET (foobar->panel);
+
+	panel_widget_save_key_event (panel, event);
+	return GTK_WIDGET_CLASS (foobar_widget_parent_class)->key_press_event (widget, event);
+}
+
 static gboolean
 foobar_leave_notify (GtkWidget *widget,
 		     GdkEventCrossing *event)
@@ -187,22 +203,18 @@ foobar_enter_notify (GtkWidget *widget,
 		GTK_WIDGET_CLASS (foobar_widget_parent_class)->enter_notify_event (widget,
 								     event);
 
-	if (global_config.autoraise)
-		gdk_window_raise (widget->window);
-
 	return FALSE;
 }
 
 static void
-gnomecal_client (GtkWidget *w, gpointer data)
+foobar_search (GtkWidget *widget,
+	       gpointer   data)
 {
-	char *v[4] = { "gnomecal", "--view", NULL, NULL };
-	v[2] = data;
-	if(gnome_execute_async (g_get_home_dir (), 3, v) < 0)
-		panel_error_dialog("cannot_execute_gnome_calendar",
-				   _("Cannot execute the gnome calendar,\n"
-				     "perhaps it's not installed.\n"
-				     "It is in the gnome-pim package."));
+	char *argv[2] = {"gnome-search-tool", NULL};
+
+	if (gnome_execute_async (g_get_home_dir (), 1, argv) < 0)
+		panel_error_dialog ("cannot_exec_gnome-search-tool",
+				    _("Cannot execute gnome-search-tool"));
 }
 
 static void
@@ -212,7 +224,7 @@ append_actions_menu (GtkWidget *menu_bar)
 
 	menu = panel_menu_new ();
 
-	item = pixmap_menu_item_new (_("Run..."), "gnome-run.png",
+	item = pixmap_menu_item_new (_("Run Program..."), "gnome-run.png",
 				     FALSE /* force_image */);
 	gtk_tooltips_set_tip (panel_tooltips, item,
 			      _("Run applications, if you know the "
@@ -223,8 +235,20 @@ append_actions_menu (GtkWidget *menu_bar)
 			  G_CALLBACK (show_run_dialog), 0);
 	setup_internal_applet_drag (item, "RUN:NEW");
 
-	/* FIXME: search */
-	/* FIXME: shutdown or reboot */
+	if (panel_is_program_in_path  ("gnome-search-tool")) {
+		item = pixmap_menu_item_new (
+				_("Search for Files..."), "gnome-searchtool.png", FALSE);
+
+		gtk_tooltips_set_tip (panel_tooltips, item,
+				      _("Find files, folders, and documents "
+				        "on your computer"),
+				      NULL);
+
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		g_signal_connect (item, "activate",
+				  G_CALLBACK (foobar_search), 0);
+	}
+
 
 	if (panel_is_program_in_path  ("xscreensaver")) {
 		item = pixmap_menu_item_new (_("Lock Display"), 
@@ -250,6 +274,8 @@ append_actions_menu (GtkWidget *menu_bar)
 			  G_CALLBACK (panel_quit), 0);
 	setup_internal_applet_drag (item, "LOGOUT:NEW");
 
+	/* FIXME: shutdown or reboot */
+
 	item = gtk_menu_item_new_with_label (_("Actions"));
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), item);
@@ -257,376 +283,6 @@ append_actions_menu (GtkWidget *menu_bar)
 	panel_stretch_events_to_toplevel (item, PANEL_STRETCH_TOP);
 }
 
-#if 0
-/* sure we won't need this????
- * -George */
-static GtkWidget *
-append_folder_menu (GtkWidget *menu_bar, const char *label,
-		    const char *pixmap, const char *path,
-		    gboolean stretch_left,
-		    gboolean stretch_top,
-		    gboolean stretch_right)
-{
-	GtkWidget *item, *menu;
-
-	menu = create_fake_menu_at (path,
-				    FALSE /* launcher_add */,
-				    label /* dir_name */,
-				    NULL /* pixmap_name */,
-				    FALSE /* title */);
-
-	if (menu == NULL) {
-		g_warning (_("menu wasn't created"));
-		return NULL;
-	}
-
-	if (pixmap != NULL)
-		item = pixmap_menu_item_new (label, pixmap,
-					     FALSE /* force_image */);
-	else
-		item = gtk_menu_item_new_with_label (label);
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), item);
-
-	g_signal_connect (G_OBJECT (menu), "show",
-			  G_CALLBACK (submenu_to_display),
-			  NULL);
-
-	if (stretch_left || stretch_top || stretch_right) {
-		PanelStretchFlags flags = PANEL_STRETCH_NONE;
-
-		flags |= stretch_left  ? PANEL_STRETCH_LEFT  : 0;
-		flags |= stretch_top   ? PANEL_STRETCH_TOP   : 0;
-		flags |= stretch_right ? PANEL_STRETCH_RIGHT : 0;
-
-		panel_stretch_events_to_toplevel (item, flags);
-	}
-		
-	return menu;
-}
-#endif
-
-static void
-append_gnomecal_items (GtkWidget *menu)
-{
-	GtkWidget *item;
-	int i;
-	
-	const char *cals[] = { 
-		N_("Today"),      N_("View the calendar for today."),      "gnome-day.png",   "dayview",
-		N_("This Week"),  N_("View the calendar for this week."),  "gnome-week.png",  "weekview",
-		N_("This Month"), N_("View the calendar for this month."), "gnome-month.png", "monthview",
-		NULL
-	};
-	
-	for (i=0; cals[i]; i+=4) {
-		item = pixmap_menu_item_new (_(cals[i]), cals[i+2],
-					     FALSE /* force_image */);
-		gtk_tooltips_set_tip (panel_tooltips, item,
-			      	      _(cals[i+1]),
-			      	      NULL);
-	
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		g_signal_connect (G_OBJECT (item), "activate",
-				  G_CALLBACK (gnomecal_client),
-				  (char *)cals[i+3]);
-	}
-}
-
-static void
-update_clock (FoobarWidget *foo)
-{
-	static int day = 0;
-	struct tm *das_tm;
-	time_t das_time;
-	char hour[256];
-	PangoLayout *layout;
-	char *str_utf8;
-	int width;
-
-	if (foo->clock_label == NULL)
-		return;
-
-	time (&das_time);
-	das_tm = localtime (&das_time);
-
-	if (das_tm->tm_mday != day) {
-		if(strftime(hour, sizeof(hour), _("%A %B %d"), das_tm) == 0) {
-			/* according to docs, if the string does not fit, the
-			 * contents of hour are undefined, thus just use
-			 * ??? */
-			strcpy(hour, "???");
-		}
-		hour[sizeof(hour)-1] = '\0'; /* just for sanity */
-
-		str_utf8 = g_locale_to_utf8 (hour, -1, NULL, NULL, NULL);
-
-		gtk_tooltips_set_tip (panel_tooltips, foo->clock_ebox,
-				      sure_string (str_utf8), NULL);
-		g_free (str_utf8);
-
-		day = das_tm->tm_mday;
-	}
-
-	if(strftime(hour, sizeof(hour), foo->clock_format, das_tm) == 0) {
-		/* according to docs, if the string does not fit, the
-		 * contents of hour are undefined, thus just use
-		 * ??? */
-		strcpy(hour, "???");
-	}
-	hour[sizeof(hour)-1] = '\0'; /* just for sanity */
-
-	str_utf8 = g_locale_to_utf8 (hour, -1, NULL, NULL, NULL);
-
-	if (str_utf8 != NULL) {
-		layout = gtk_widget_create_pango_layout (foo->clock_label, str_utf8);
-		pango_layout_get_pixel_size (layout, &width, NULL);
-		width += 8; /* Padding */
-		g_object_unref (G_OBJECT(layout));
-
-		if (width > foo->clock_label->requisition.width)
-			gtk_widget_set_size_request (foo->clock_label, width, -1);
-
-		gtk_label_set_text (GTK_LABEL (foo->clock_label), str_utf8);
-
-		g_free (str_utf8);
-	}
-}
-
-static int
-timeout_cb (gpointer data)
-{
-	FoobarWidget *foo = FOOBAR_WIDGET (data);
-
-	if (foo->clock_label == NULL) {
-		foo->clock_timeout = 0;
-		return FALSE;
-	}
-
-	update_clock (foo);
-
-	return TRUE;
-}
-
-static void
-set_fooclock_format (GtkWidget *w, const char *format)
-{
-	if ( ! GTK_CHECK_MENU_ITEM (w)->active)
-		return;
-
-	foobar_widget_global_set_clock_format (format);
-}
-
-static void
-append_format_items (FoobarWidget *foo,
-		     GtkWidget    *menu)
-{
-	char hour[256];
-	GSList *group = NULL;
-	struct tm *das_tm;
-	time_t das_time = 0;
-	const char *key;
-	char *str_utf8;
-	char *s;
-	int i;
-	const char *formats[FORMAT_ITEM_NUM] = {
-		N_("%H:%M"),
-		N_("%H:%M:%S"),
-		N_("%l:%M %p"),
-		N_("%l:%M:%S %p")
-	};
-
-	key = panel_gconf_full_key (
-			PANEL_GCONF_PANELS,
-			panel_gconf_get_profile (),
-			PANEL_WIDGET (foo->panel)->unique_id,
-			"clock-format");
-	s = panel_gconf_get_string (key, _("%I:%M:%S %p"));
-	
-	for (i = 0; i < FORMAT_ITEM_NUM; i++) {
-		das_tm = localtime (&das_time);
-		if (strftime (hour, sizeof(hour), _(formats[i]), das_tm) == 0) {
- 			/* according to docs, if the string does not fit, the
- 		 	 * contents of hour are undefined, thus just use
- 		 	 * ??? */
-			strcpy(hour, "???");
-		}
-		hour[sizeof(hour)-1] = '\0'; /* just for sanity */
-
-		str_utf8 = g_locale_to_utf8 (hour, -1, NULL, NULL, NULL);
-		format_items[i] = gtk_radio_menu_item_new_with_label (group, sure_string (str_utf8));
-		g_object_set_data (G_OBJECT (format_items[i]),
-				   "Format", _(formats[i]));
-
-		g_free (str_utf8);
-
-		group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (format_items[i]));
-	
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), format_items[i]);	
-		g_signal_connect (G_OBJECT (format_items[i]), "activate",
-				  G_CALLBACK (set_fooclock_format),
-				  _(formats[i]));
-
-		if (!strcmp (sure_string (s), _(formats[i])))
-			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (format_items[i]), TRUE);
-	}
-
-	g_free (s);
-}
-
-static void
-set_time_cb (GtkWidget *menu_item, char *path)
-{
-	char *v[2] = { path };
-	
-	if (gnome_execute_async (g_get_home_dir (), 1, v) < 0)
-		panel_error_dialog ("could_not_call_time_admin",
-				    _("Could not call time-admin\n"
-				      "Perhaps time-admin is not installed"));
-}
-
-static GtkWidget *
-append_clock_menu (FoobarWidget *foo, GtkWidget *menu_bar)
-{
-	GtkWidget *item, *menu, *menu2;
-	gchar *time_admin_path;
-
-	menu = panel_menu_new ();
-	append_gnomecal_items (menu);
-
-#if 0 /* put back when evolution can do this */
-	item = gtk_image_menu_item_new_with_label (_("Add Appointment..."));
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-#endif
-
-	add_menu_separator (menu);
-
-	/* check for time-admin (part of ximian-setup-tools) */
-	time_admin_path = g_find_program_in_path  ("time-admin");
-	if (time_admin_path != NULL) {
-		item = pixmap_menu_item_new (_("Set Time..."), "gnome-set-time.png",
-					     FALSE /* force_image */);
-		gtk_tooltips_set_tip (panel_tooltips, item,
-			      	      _("Adjust the date and time."),
-			      	      NULL);	
-		
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		g_signal_connect (G_OBJECT (item), "activate",
-				  G_CALLBACK (set_time_cb),
-				  time_admin_path);			
-	}
-
-	menu2 = panel_menu_new ();
-	append_format_items (foo, menu2); 
-
-	item = gtk_image_menu_item_new_with_label (_("Format"));
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu2);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-
-	item = gtk_menu_item_new ();
-
-	foo->clock_label = gtk_label_new ("");
-	foo->clock_timeout = g_timeout_add (1000, timeout_cb, foo);
-
-	foo->clock_ebox = item;
-	gtk_container_add (GTK_CONTAINER (item), foo->clock_label);
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
-
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), item);
-
-	panel_stretch_events_to_toplevel (item, PANEL_STRETCH_TOP);
-
-	return item;
-}
-
-void
-foobar_widget_global_set_clock_format (const char *format)
-{
-	GList *li;
-
-	for (li = foobars; li != NULL; li = li->next) {
-		foobar_widget_set_clock_format (FOOBAR_WIDGET (li->data),
-						format);
-	}
-}
-
-/*
- * FIXME: we should be only saving Foobar specific prefs
- */
-static void
-foobar_save_to_gconf (FoobarWidget *foo)
-{
-	PanelData *pd;
-
-	pd = g_object_get_data (G_OBJECT (foo), "PanelData");
-
-	if (pd)
-		panel_save_to_gconf (pd);
-}
-
-void
-foobar_widget_set_clock_format (FoobarWidget *foo, const char *clock_format)
-{
-	int i;
-
-	g_return_if_fail (FOOBAR_IS_WIDGET (foo));
-
-	if (foo->clock_format != NULL)
-		g_free (foo->clock_format);
-	foo->clock_format = g_strdup (clock_format);
-
-	if(foo->clock_label) {
-		time_t das_time;
-		struct tm *das_tm;
-		char hour[256];
-		gchar *str_utf8;
-		int width;
-		PangoLayout *layout;
-
-		das_time = 0;
-		das_tm = localtime (&das_time);
-		if (strftime (hour, sizeof(hour), _(clock_format), das_tm) == 0) {
-			/* according to docs, if the string does not fit, the
-			 * contents of hour are undefined, thus just use
-			 * ??? */
-			strcpy(hour, "???");
-		}
-		hour[sizeof(hour)-1] = '\0'; /* just for sanity */
-
-		str_utf8 = g_locale_to_utf8 (hour, -1, NULL, NULL, NULL);
-		if (str_utf8 != NULL) {
-			layout = gtk_widget_create_pango_layout (foo->clock_label, str_utf8);
-			pango_layout_get_pixel_size (layout, &width, NULL);
-			width += 8; /* Padding */
-			gtk_widget_set_size_request (foo->clock_label, width, -1);
-			g_object_unref (G_OBJECT(layout));
-		}
-		g_free (str_utf8);
-	}
-
-	/* update the menu */
-	for (i = 0; i < FORMAT_ITEM_NUM; i++) {
-		const char *item_format;
-
-		/* sanity (or insanity?) */
-		if (format_items[i] == NULL)
-			break;
-
-		item_format = g_object_get_data (G_OBJECT (format_items[i]), "Format");
-		if (strcmp (sure_string (item_format), sure_string (clock_format)) == 0) {
-			if ( ! GTK_CHECK_MENU_ITEM (format_items[i])->active)
-				gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (format_items[i]), TRUE);
-			break;
-		} else if (GTK_CHECK_MENU_ITEM (format_items[i])->active) {
-			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (format_items[i]), FALSE);
-		}
-	}
-
-	update_clock (foo);
-
-	foobar_save_to_gconf (foo);
-}
-
 void
 foobar_widget_update_winhints (FoobarWidget *foo)
 {
@@ -659,20 +315,16 @@ foobar_widget_realize (GtkWidget *w)
 				 0 /* bottom */);
 }
 
+
 static void
 programs_menu_to_display (GtkWidget *menu)
 {
 	if (menu_need_reread (menu)) {
-		int flags;
-
 		while (GTK_MENU_SHELL (menu)->children)
 			gtk_widget_destroy (GTK_MENU_SHELL (menu)->children->data);
-		flags = MAIN_MENU_SYSTEM;
-		if (got_kde_menus ())
-			flags |= MAIN_MENU_KDE_SUB;
-		if (got_distro_menus ())
-			flags |= MAIN_MENU_DISTRIBUTION_SUB;
-		create_root_menu (menu, TRUE, flags, FALSE, FALSE /* run_item */);
+
+		create_root_menu (menu, TRUE, FOOBAR_MENU_FLAGS,
+				  FALSE, FALSE /* run_item */);
 	}
 }
 
@@ -695,13 +347,9 @@ set_the_task_submenu (FoobarWidget *foo, GtkWidget *item)
 static void
 focus_window (GtkWidget *w, WnckWindow *window)
 {
-	WnckScreen *screen = wnck_screen_get (0 /* FIXME screen number */);
-	WnckWorkspace *wspace = wnck_screen_get_active_workspace (screen);
-
-	if (wspace != NULL)
-		wnck_window_move_to_workspace (window, wspace);
 	if (wnck_window_is_minimized (window)) 
 		wnck_window_unminimize (window);
+
 	wnck_window_activate (window);
 }
 
@@ -845,12 +493,6 @@ create_task_menu (GtkWidget *w, gpointer data)
 
 	if (GTK_WIDGET_VISIBLE (foo->task_menu))
 		our_gtk_menu_position (GTK_MENU (foo->task_menu));
-
-	/* Owen: don't read the next line */
-#if 0
-	GTK_MENU_SHELL (foo->task_menu)->active = 1;
-	our_gtk_menu_position (GTK_MENU (foo->task_menu));
-#endif
 }
 
 static void
@@ -1038,13 +680,16 @@ setup_task_menu (FoobarWidget *foo)
 static void
 foobar_widget_instance_init (FoobarWidget *foo)
 {
-	char *path;
-	GtkWindow *window = GTK_WINDOW (foo);
+	GtkWindow *window;
 	GtkWidget *bufmap;
-	GtkWidget *menu_bar, *bar;
-	GtkWidget *menu, *menuitem;
+	GtkWidget *menu_bar;
+	GtkWidget *task_bar;
+	GtkWidget *menu;
+	GtkWidget *menuitem;
 	GtkWidget *align;
-	gint flags;
+	char      *path;
+
+	window = GTK_WINDOW (foo);
 
 	foo->screen = 0;
 
@@ -1055,10 +700,6 @@ foobar_widget_instance_init (FoobarWidget *foo)
 	foo->task_bin   = NULL;
 	foo->icon_window = NULL;
 
-	foo->clock_format = g_strdup (_("%H:%M"));
-	foo->clock_timeout = 0;
-	foo->clock_label = NULL;
-
 	foo->compliant_wm = xstuff_is_compliant_wm ();
 	if(foo->compliant_wm)
 		GTK_WINDOW(foo)->type = GTK_WINDOW_TOPLEVEL;
@@ -1098,13 +739,9 @@ foobar_widget_instance_init (FoobarWidget *foo)
 	menuitem = pixmap_menu_item_new (_("Applications"),
 					 "gnome-logo-icon-transparent.png",
 					 TRUE /* force_image */);
-	flags = MAIN_MENU_SYSTEM;
-	if (got_kde_menus ())
-		flags |= MAIN_MENU_KDE_SUB;
-	if (got_distro_menus ())
-		flags |= MAIN_MENU_DISTRIBUTION_SUB;
 
-	menu = create_root_menu (NULL, TRUE, flags, FALSE, FALSE /* run_item */);
+	menu = create_root_menu (NULL, TRUE, FOOBAR_MENU_FLAGS,
+				 FALSE, FALSE /* run_item */);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menu);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), menuitem);
 	g_signal_connect (G_OBJECT (menu), "show",
@@ -1132,6 +769,8 @@ foobar_widget_instance_init (FoobarWidget *foo)
 
 	g_object_set_data (G_OBJECT (menu_bar), "menu_panel", foo->panel);
 
+	add_atk_name_desc (foo->panel, _("Base Panel"), _("Gnome Base Panel"));
+
 	path = panel_pixmap_discovery ("panel-corner-right.png", FALSE /* fallback */);
 	if (path != NULL) {
 		bufmap = gtk_image_new_from_file (path);
@@ -1141,17 +780,14 @@ foobar_widget_instance_init (FoobarWidget *foo)
 		gtk_box_pack_end (GTK_BOX (foo->hbox), align, FALSE, FALSE, 0);
 	}
 
-	bar = menu_bar = gtk_menu_bar_new ();
-	gtk_widget_set_name (menu_bar,
-			     "panel-foobar-menubar");
+	task_bar = gtk_menu_bar_new ();
+	gtk_widget_set_name (task_bar, "panel-foobar-menubar");
 
-	append_clock_menu (foo, menu_bar);
+	append_task_menu (foo, GTK_MENU_SHELL (task_bar));
 
-	append_task_menu (foo, GTK_MENU_SHELL (bar));
-
-
-	gtk_box_pack_end (GTK_BOX (foo->hbox), menu_bar, FALSE, FALSE, 0);
+	gtk_box_pack_end (GTK_BOX (foo->hbox), task_bar, FALSE, FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (foo), foo->ebox);
+
 	gtk_widget_show_all (foo->ebox);
 }
 
@@ -1178,15 +814,6 @@ foobar_widget_destroy (GtkObject *o)
 
 	foobars = g_list_remove (foobars, foo);
 
-	if (foo->clock_timeout != 0)
-		g_source_remove (foo->clock_timeout);
-	foo->clock_timeout = 0;
-	
-	foo->clock_label = NULL;
-
-	g_free (foo->clock_format);
-	foo->clock_format = NULL;
-
 	g_slist_foreach (panel_list, queue_panel_resize, NULL);
 
 	if (foo->windows != NULL)
@@ -1226,7 +853,6 @@ GtkWidget *
 foobar_widget_new (const char *panel_id, int screen)
 {
 	FoobarWidget *foo;
-	char *panel_icon;
 
 	g_return_val_if_fail (screen >=0, NULL);
 
@@ -1235,14 +861,6 @@ foobar_widget_new (const char *panel_id, int screen)
 
 	foo = g_object_new (FOOBAR_TYPE_WIDGET, NULL);
 
-        panel_icon = panel_pixmap_discovery ("gnome-panel.png",
-                                             FALSE);
-
-        if (panel_icon) { 
-                gnome_window_icon_set_default_from_file (panel_icon);
-                g_free (panel_icon);
-        }
-
 	if (panel_id)
 		panel_widget_set_id (PANEL_WIDGET (foo->panel), panel_id);
 
@@ -1415,5 +1033,3 @@ foobar_widget_move_focus_out (FoobarWidget *foobar)
 {
  	panel_widget_focus (PANEL_WIDGET (foobar->panel));
 }
-
-
