@@ -27,6 +27,7 @@
 #include "rgb-stuff.h"
 #include "panel-typebuiltins.h"
 #include "drawer-widget.h"
+#include "panel-applet-frame.h"
 
 #define MOVE_INCREMENT 2
 
@@ -229,6 +230,7 @@ add_all_move_bindings (PanelWidget *panel)
 	add_move_bindings (binding_set, GDK_SHIFT_MASK, "push_move");
 	add_move_bindings (binding_set, GDK_CONTROL_MASK, "switch_move");
 	add_move_bindings (binding_set, GDK_MOD1_MASK, "free_move");
+	add_move_bindings (binding_set, 0, "free_move");
 
 	add_tab_bindings (binding_set, 0, TRUE);
 	add_tab_bindings (binding_set, GDK_SHIFT_MASK, FALSE);
@@ -236,6 +238,18 @@ add_all_move_bindings (PanelWidget *panel)
 	gtk_binding_entry_add_signal (binding_set,
                                       GDK_Escape, 0,
                                       "end_move", 0);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_KP_Enter, 0,
+                                      "end_move", 0);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_Return, 0,
+                                      "end_move", 0);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_KP_Space, 0,
+                                      "end_move", 0);
+	gtk_binding_entry_add_signal (binding_set,
+                                      GDK_space, 0,
+                                      "end_move", 0);
 }
 
 static void
@@ -271,10 +285,15 @@ remove_all_move_bindings (PanelWidget *panel)
 	remove_move_bindings (binding_set, GDK_SHIFT_MASK);
 	remove_move_bindings (binding_set, GDK_CONTROL_MASK);
 	remove_move_bindings (binding_set, GDK_MOD1_MASK);
+	remove_move_bindings (binding_set, 0);
 	remove_tab_bindings (binding_set, 0, TRUE);
 	remove_tab_bindings (binding_set, GDK_SHIFT_MASK, FALSE);
 
 	gtk_binding_entry_clear (binding_set, GDK_Escape, 0);
+	gtk_binding_entry_clear (binding_set, GDK_KP_Enter, 0);
+	gtk_binding_entry_clear (binding_set, GDK_Return, 0);
+	gtk_binding_entry_clear (binding_set, GDK_KP_Space, 0);
+	gtk_binding_entry_clear (binding_set, GDK_space, 0);
 }
 
 static void
@@ -645,6 +664,27 @@ panel_widget_is_applet_stuck(PanelWidget *panel, GtkWidget *applet)
 	return FALSE;
 }
 
+static int
+get_size_from_hints (AppletData *ad, int cells)
+{
+	int i;
+	
+	for (i = 0; i < ad->size_hints_len; i += 2) {
+		if (cells > ad->size_hints[i]) {
+			/* Clip to top */
+			cells = ad->size_hints[i];
+			break;
+		}
+		if (cells <= ad->size_hints[i] &&
+		    cells >= ad->size_hints[i+1]) {
+			/* Keep cell size */
+			break;
+		}
+	}
+	
+	return cells;
+}
+
 static int
 allocate_dirty_child(gpointer data)
 {
@@ -689,13 +729,22 @@ allocate_dirty_child(gpointer data)
 
 	if (!panel->packed && ad->expand_major) {
 		GList *l = g_list_find (panel->applet_list, ad);
+		int cells;
+		
 		if (l) {
 			if (l->next) {
 				nad = l->next->data;
-				ad->cells = nad->pos - ad->pos - 1;
+				cells = nad->pos - ad->pos - 1;
 			} else {
-				ad->cells = panel->size - ad->pos - 1;
+				cells = panel->size - ad->pos - 1;
 			}
+
+			if (ad->size_hints) 
+				cells = get_size_from_hints (ad, cells);
+			
+			cells = MAX (cells, ad->min_cells);
+			
+			ad->cells = cells;
 			
 			if (panel->orient == GTK_ORIENTATION_HORIZONTAL) {
 				challoc.width = ad->cells;
@@ -1045,6 +1094,14 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 		AppletData *ad = list->data;
 		GtkRequisition chreq;
 		gtk_widget_size_request(ad->applet,&chreq);
+
+		if (ad->expand_major && PANEL_IS_APPLET_FRAME (ad->applet)) {
+			if (ad->size_hints != NULL)
+				g_free (ad->size_hints);
+			ad->size_hints = panel_applet_frame_get_size_hints (PANEL_APPLET_FRAME (ad->applet),
+									    &ad->size_hints_len);
+		}
+		
 		if(panel->orient == GTK_ORIENTATION_HORIZONTAL) {
 			if(requisition->height < chreq.height)
 				requisition->height = chreq.height;
@@ -1273,6 +1330,8 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 		    list = g_list_previous(list)) {
 			AppletData *ad = list->data;
 			GtkRequisition chreq;
+			int cells;
+			
 			gtk_widget_get_child_requisition(ad->applet,&chreq);
 
 			if(panel->orient == GTK_ORIENTATION_HORIZONTAL)
@@ -1283,7 +1342,13 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 			ad->min_cells = ad->cells;
 			
 			if (ad->expand_major) {
-				ad->cells = (i - ad->pos) - 1;
+				cells = (i - ad->pos) - 1;
+
+				if (ad->size_hints) 
+					cells = get_size_from_hints (ad, cells);
+				cells = MAX (cells, ad->min_cells);
+
+				ad->cells = cells;
 			}
 
 			if(ad->pos+ad->min_cells > i) {
@@ -1633,6 +1698,9 @@ panel_widget_finalize (GObject *obj)
 	g_free (panel->unique_id);
 	panel->unique_id = NULL;
 
+	if (panel->key_event)
+		gdk_event_free ((GdkEvent *)panel->key_event);
+
 	if (G_OBJECT_CLASS (panel_widget_parent_class)->finalize)
 		G_OBJECT_CLASS (panel_widget_parent_class)->finalize (obj);
 }
@@ -2427,6 +2495,8 @@ panel_widget_applet_destroy (GtkWidget *applet, gpointer data)
 		panel->applet_list = g_list_remove (panel->applet_list,ad);
 	}
 
+	g_free (ad->size_hints);
+
 	g_free (ad);
 }
 
@@ -2569,6 +2639,7 @@ panel_widget_add (PanelWidget *panel,
 		ad->no_die = 0;
 		ad->expand_major = expand_major;
 		ad->expand_minor = expand_minor;
+		ad->size_hints = NULL;
 		g_object_set_data (G_OBJECT (applet),
 				   PANEL_APPLET_DATA, ad);
 		
@@ -2863,7 +2934,9 @@ panel_widget_focus_in_event (GtkWidget *widget, GdkEventFocus *event)
 static gboolean
 panel_widget_focus_out_event (GtkWidget *widget, GdkEventFocus *event)
 {
-	if (GTK_WINDOW (PANEL_WIDGET (widget)->panel_parent)->has_focus)
+	PanelWidget *panel_widget = PANEL_WIDGET (widget);
+
+	if (GTK_WINDOW (panel_widget->panel_parent)->has_focus)
 		GTK_WIDGET_UNSET_FLAGS (widget, GTK_CAN_FOCUS);
 
 	GTK_WIDGET_CLASS (panel_widget_parent_class)->focus_out_event (widget, event);
@@ -2875,9 +2948,22 @@ static gboolean
 panel_widget_real_focus (GtkWidget        *widget,
                          GtkDirectionType  direction)
 {
-	if (GTK_WIDGET_HAS_FOCUS (widget))
-		GTK_WIDGET_UNSET_FLAGS (widget, GTK_CAN_FOCUS);
+	PanelWidget *panel = PANEL_WIDGET (widget);
 
+	if (panel->key_event) {
+		if (panel->key_event->state & GDK_CONTROL_MASK) {
+			/*
+	 		 * Ctrl+Tab was pressed when focus was in applet in
+			 * another process
+			 */
+			panel_widget_focus (PANEL_WIDGET (widget));
+			return TRUE;
+		}
+	}
+
+	if (GTK_WIDGET_HAS_FOCUS (widget)) {
+		GTK_WIDGET_UNSET_FLAGS (widget, GTK_CAN_FOCUS);
+	}
 	return GTK_WIDGET_CLASS (panel_widget_parent_class)->focus (widget, direction);
 }
 
@@ -2907,3 +2993,14 @@ panel_widget_get_applet_orient (PanelWidget *panel)
 
 	return basep_widget_get_applet_orient (BASEP_WIDGET (panel->panel_parent));
 }
+
+void 
+panel_widget_save_key_event (PanelWidget *panel,
+			     GdkEventKey *key_event)
+{
+	if (panel->key_event) {
+		gdk_event_free ((GdkEvent *)panel->key_event);
+	}
+	panel->key_event = (GdkEventKey *) gdk_event_copy ((GdkEvent *)key_event);
+}
+
